"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = exports.createDenyActionMiddleware = exports.allowCommandMiddleware = void 0;

var _lodash = _interopRequireDefault(require("lodash"));

var _enum = require("../enum");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var _parseActionList = function _parseActionList(action) {
  var resultActions = new Set();
  var actionList = _lodash.default.isArray(action) ? action : [action];

  _lodash.default.each(actionList, function (a) {
    if (_lodash.default.isString(a)) {
      switch (a) {
        case _enum.EntityAction.CREATE:
        case _enum.EntityAction.LOAD_BY_ID:
        case _enum.EntityAction.LOAD_ALL:
        case _enum.EntityAction.LOAD_BY_QUERY:
        case _enum.EntityAction.UPDATE_BY_ID:
        case _enum.EntityAction.UPDATE:
        case _enum.EntityAction.REMOVE_BY_ID:
        case _enum.EntityAction.REMOVE_BY_QUERY:
        case _enum.EntityAction.RUN_COMMAND:
          resultActions.add(a);
          break;

        case 'create':
          resultActions.add(_enum.EntityAction.CREATE);
          break;

        case 'read':
          resultActions.add(_enum.EntityAction.LOAD_BY_ID);
          resultActions.add(_enum.EntityAction.LOAD_ALL);
          resultActions.add(_enum.EntityAction.LOAD_BY_QUERY);
          break;

        case 'update':
          resultActions.add(_enum.EntityAction.UPDATE_BY_ID);
          resultActions.add(_enum.EntityAction.UPDATE);
          break;

        case 'remove':
        case 'delete':
          resultActions.add(_enum.EntityAction.REMOVE_BY_ID);
          resultActions.add(_enum.EntityAction.REMOVE_BY_QUERY);
          break;

        case 'write':
          resultActions.add(_enum.EntityAction.CREATE);
          resultActions.add(_enum.EntityAction.UPDATE_BY_ID);
          resultActions.add(_enum.EntityAction.UPDATE);
          resultActions.add(_enum.EntityAction.REMOVE_BY_ID);
          resultActions.add(_enum.EntityAction.REMOVE_BY_QUERY);
          break;

        case 'all':
        case '*':
          resultActions.add(_enum.EntityAction.CREATE);
          resultActions.add(_enum.EntityAction.LOAD_BY_ID);
          resultActions.add(_enum.EntityAction.LOAD_ALL);
          resultActions.add(_enum.EntityAction.LOAD_BY_QUERY);
          resultActions.add(_enum.EntityAction.UPDATE_BY_ID);
          resultActions.add(_enum.EntityAction.UPDATE);
          resultActions.add(_enum.EntityAction.REMOVE_BY_ID);
          resultActions.add(_enum.EntityAction.REMOVE_BY_QUERY);
          break;

        case 'command':
          resultActions.add(_enum.EntityAction.RUN_COMMAND);
          break;
      }
    }
  });

  return Array.from(resultActions);
};

var createDenyActionMiddleware = function createDenyActionMiddleware(backend, actionList) {
  var denyList = _parseActionList(actionList);

  var denyActionArray = _lodash.default.isArray(denyList) ? denyList : [denyList];
  return function (req, res, next) {
    var _backend$parseRequest = backend.parseRequest(req),
        action = _backend$parseRequest.action;

    if (denyActionArray.indexOf(action) !== -1) {
      return next(new Error("Permission denied for EntityAction ".concat(action)));
    }

    next();
  };
};

exports.createDenyActionMiddleware = createDenyActionMiddleware;

var allowCommandMiddleware = function allowCommandMiddleware(backend, commandList) {
  var commandArray = _lodash.default.isArray(commandList) ? commandList : [commandList];
  return function (req, res, next) {
    var _backend$parseRequest2 = backend.parseRequest(req),
        action = _backend$parseRequest2.action,
        payload = _backend$parseRequest2.payload;

    if (action === _enum.EntityAction.RUN_COMMAND) {
      /* { command, data } */
      var command = payload.command;

      if (commandArray.indexOf(command) === -1) {
        return next(new Error("Permission denied for command '".concat(command, "'")));
      }
    }

    next();
  };
};

exports.allowCommandMiddleware = allowCommandMiddleware;
var _default = {
  allowCommandMiddleware: allowCommandMiddleware,
  createDenyActionMiddleware: createDenyActionMiddleware
};
exports.default = _default;