"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.getPermittedFields = getPermittedFields;
exports.checkPermissionWithData = checkPermissionWithData;
exports.rulesDontNeedData = exports.checkRule = exports.flattenObject = exports.difference = exports.deepestClone = void 0;

var _lodash = _interopRequireDefault(require("lodash"));

var _enum = require("../enum");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _toConsumableArray(arr) { return _arrayWithoutHoles(arr) || _iterableToArray(arr) || _nonIterableSpread(); }

function _nonIterableSpread() { throw new TypeError("Invalid attempt to spread non-iterable instance"); }

function _iterableToArray(iter) { if (Symbol.iterator in Object(iter) || Object.prototype.toString.call(iter) === "[object Arguments]") return Array.from(iter); }

function _arrayWithoutHoles(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = new Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } }

var deepestClone = function deepestClone(obj) {
  try {
    return JSON.parse(JSON.stringify(obj));
  } catch (err) {
    return null;
  }
};

exports.deepestClone = deepestClone;

var difference = function difference(object, base) {
  var changes = function changes(object, base) {
    var inner = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : false;
    var changed = false;
    return _lodash.default.transform(object, function (result, value, key) {
      if (!_lodash.default.isEqual(value, base[key])) {
        changed = true;
        result[key] = _lodash.default.isObject(value) && _lodash.default.isObject(base[key]) ? changes(value, base[key], true) : value;
      }

      if (changed && inner) {
        Object.assign(result, object);
      }
    });
  };

  var diff = changes(object, base);

  _lodash.default.each(diff, function (val, key) {
    if (_lodash.default.isArray(val)) diff[key] = object[key];
  });

  return diff;
};

exports.difference = difference;

var flattenObject = function flattenObject(_obj) {
  var obj = deepestClone(_obj);

  _lodash.default.each(obj, function (val, key) {
    if (_lodash.default.isObject(val) && !_lodash.default.isArray(val) && !_lodash.default.isFunction(val)) {
      _lodash.default.each(val, function (_val, _key) {
        obj[_key] = _val;
      });

      delete obj[key];
    }
  });

  return obj;
};

exports.flattenObject = flattenObject;

var checkRule = function checkRule(_ref) {
  var rule = _ref.rule,
      auth = _ref.auth,
      newData = _ref.newData,
      data = _ref.data,
      collection = _ref.collection,
      tag = _ref.tag,
      action = _ref.action;
  var actionIsPermitted = !_lodash.default.isEmpty(rule.permittedActions) ? rule.permittedActions.indexOf(action) > -1 : true;
  var actionIsForbidden = !_lodash.default.isEmpty(rule.forbiddenActions) ? rule.forbiddenActions.indexOf(action) > -1 : false;
  if (actionIsForbidden || !actionIsPermitted) return false;
  return eval(_createEvaluationString({
    data: data,
    newData: newData,
    rule: rule,
    auth: auth,
    collection: collection,
    tag: tag,
    action: action
  }));
};

exports.checkRule = checkRule;

function _createEvaluationString(_ref2) {
  var rule = _ref2.rule,
      auth = _ref2.auth,
      newData = _ref2.newData,
      data = _ref2.data,
      action = _ref2.action;
  return "let action = ".concat(JSON.stringify(action), "; let auth = ").concat(JSON.stringify(auth), ";let newData = ").concat(newData ? JSON.stringify(newData) : undefined, "; let data = ").concat(data ? JSON.stringify(data) : undefined, "; ").concat(rule);
}

function getPermittedFields(rules, schema) {
  var biggestRule = [];
  rules.forEach(function (rule) {
    var include = rule.include,
        exclude = rule.exclude;
    if (!include && !exclude) include = _enum.ALL_COLLECTIONS;
    var allKeys = Object.keys(schema);
    if (include === _enum.ALL_COLLECTIONS) include = allKeys;
    if (exclude === _enum.ALL_COLLECTIONS) exclude = allKeys;
    if (include) exclude = _lodash.default.without.apply(_lodash.default, [allKeys].concat(_toConsumableArray(include)));else if (exclude) include = _lodash.default.without.apply(_lodash.default, [allKeys].concat(_toConsumableArray(exclude)));

    var fields = _lodash.default.without.apply(_lodash.default, [include].concat(_toConsumableArray(exclude)));

    biggestRule = [].concat(_toConsumableArray(biggestRule), _toConsumableArray(fields));
  });
  return Array.from(new Set(biggestRule));
}

function checkPermissionWithData(_ref3) {
  var data = _ref3.data,
      action = _ref3.action,
      collection = _ref3.collection,
      schema = _ref3.schema,
      auth = _ref3.auth,
      newData = _ref3.newData,
      op = _ref3.op,
      rules = _ref3.rules,
      tag = _ref3.tag,
      _ref3$select = _ref3.select,
      select = _ref3$select === void 0 ? null : _ref3$select;
  data = _lodash.default.isEmpty(data) ? [null] : data;
  var status = false;
  data = _lodash.default.chain(data).map(function (elem) {
    var satisfyingRules = _lodash.default.filter(rules, function (rule) {
      return checkRule({
        newData: newData,
        data: elem,
        rule: rule[op],
        auth: auth,
        collection: collection,
        tag: tag,
        action: action
      });
    });

    status = status || !!satisfyingRules.length;
    if (!satisfyingRules.length) return null;
    if (action === _enum.EntityAction.CREATE) elem = newData;

    if (op === _enum.READ && elem) {
      var permittedKeys = getPermittedFields(satisfyingRules, schema);
      elem = _lodash.default.pick(elem, permittedKeys);
      elem = select ? _lodash.default.pick(elem, select) : elem;
    }

    return elem;
  }).compact().value();
  return {
    status: status,
    data: data
  };
}

var rulesDontNeedData = function rulesDontNeedData(rules, op) {
  var dataRe = /(;|\s|\n|\t|\(|{|\|\||&&)data/;
  return !_lodash.default.find(rules, function (rule) {
    return dataRe.test(rule[op]);
  });
};

exports.rulesDontNeedData = rulesDontNeedData;