"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = _default;

var _regenerator = _interopRequireDefault(require("@babel/runtime/regenerator"));

var _mongoBackend = _interopRequireDefault(require("../backends/mongoBackend"));

var _mongomanager = require("@sfast/mongomanager");

var _enum = require("../enum");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function asyncGeneratorStep(gen, resolve, reject, _next, _throw, key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { Promise.resolve(value).then(_next, _throw); } }

function _asyncToGenerator(fn) { return function () { var self = this, args = arguments; return new Promise(function (resolve, reject) { var gen = fn.apply(self, args); function _next(value) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "next", value); } function _throw(err) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "throw", err); } _next(undefined); }); }; }

function _default() {
  var _ref = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {},
      backend = _ref.backend,
      oplogUrl = _ref.oplogUrl,
      oplogNamespace = _ref.oplogNamespace,
      _ref$acceptDbs = _ref.acceptDbs,
      acceptDbs = _ref$acceptDbs === void 0 ? [] : _ref$acceptDbs,
      _ref$ignoreDbs = _ref.ignoreDbs,
      ignoreDbs = _ref$ignoreDbs === void 0 ? [] : _ref$ignoreDbs;

  if (!(backend instanceof _mongoBackend.default)) {
    throw new Error('For oplog watch plugin, backend must be instanceOf MongoBackend');
  }

  if (oplogNamespace) {
    oplogNamespace = oplogNamespace.split('.');

    if (oplogNamespace[0] !== '*' && oplogNamespace[1] === '*') {
      oplogNamespace = oplogNamespace.join('\\.');
    } else {}
  }

  var mongoOplog = new _mongomanager.MongoOplog({
    oplogurl: oplogUrl,
    namespace: oplogNamespace
  });
  mongoOplog.on(
  /* OplogEvent.OPERATION */
  'op', _handleOplog);
  mongoOplog.start();

  function _handleOplog(_x) {
    return _handleOplog2.apply(this, arguments);
  }

  function _handleOplog2() {
    _handleOplog2 = _asyncToGenerator(
    /*#__PURE__*/
    _regenerator.default.mark(function _callee(oplogData) {
      var op, db, collection, id, data, Entity, idField, entity;
      return _regenerator.default.wrap(function _callee$(_context) {
        while (1) {
          switch (_context.prev = _context.next) {
            case 0:
              op = oplogData.op, db = oplogData.db, collection = oplogData.collection, id = oplogData.id, data = oplogData.data;

              if (!(acceptDbs.length && acceptDbs.indexOf(db) === -1)) {
                _context.next = 3;
                break;
              }

              return _context.abrupt("return");

            case 3:
              if (!(ignoreDbs.length && ignoreDbs.indexOf(db) !== -1)) {
                _context.next = 5;
                break;
              }

              return _context.abrupt("return");

            case 5:
              _context.t0 = op;
              _context.next = _context.t0 === 'i' ? 8 : _context.t0 === 'd' ? 10 : _context.t0 === 'u' ? 12 : 23;
              break;

            case 8:
              backend.emitWatch({
                action: _enum.WatchActions.CREATE,
                data: data,
                collection: collection,
                tag: backend.tag
              });
              return _context.abrupt("break", 23);

            case 10:
              // where to find idField value
              backend.emitWatch({
                action: _enum.WatchActions.DELETE,
                data: {
                  _id: id
                },
                collection: collection,
                tag: backend.tag
              });
              return _context.abrupt("break", 23);

            case 12:
              Entity = backend.getEntity({
                tag: backend.tag,
                collection: collection
              });

              if (Entity) {
                _context.next = 15;
                break;
              }

              return _context.abrupt("return");

            case 15:
              idField = Entity.idField;
              _context.next = 18;
              return backend.loadByQuery(collection, {
                query: {
                  _id: id
                },
                select: idField
              }, {
                masterKey: backend.masterKey
              });

            case 18:
              entity = _context.sent;
              entity = entity[0];

              if (entity) {
                _context.next = 22;
                break;
              }

              return _context.abrupt("return");

            case 22:
              backend.emitWatch({
                action: _enum.WatchActions.UPDATE,
                data: _updateData({
                  id: entity[idField],
                  data: data,
                  idField: idField
                }),
                collection: collection,
                tag: backend.tag
              });

            case 23:
            case "end":
              return _context.stop();
          }
        }
      }, _callee);
    }));
    return _handleOplog2.apply(this, arguments);
  }

  function _updateData() {
    var _ref4;

    var _ref2 = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {},
        id = _ref2.id,
        data = _ref2.data,
        idField = _ref2.idField;

    if (!data.$set && !data.$unset && data._id) {
      var _ref3;

      return _ref3 = {}, _defineProperty(_ref3, idField, id), _defineProperty(_ref3, "set", data), _ref3;
    }

    return _ref4 = {}, _defineProperty(_ref4, idField, id), _defineProperty(_ref4, "update", data.$set || {}), _defineProperty(_ref4, "remove", data.$unset || {}), _ref4;
  }
}